<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet
	xmlns="http://jbg.sf.net/jbg"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:seaf="http://seaf.sf.net/seaf"
	xmlns:per="http://seaf.sf.net/seaf-persistence"
	xmlns:svc="http://seaf.sf.net/seaf-service"
	version="2.0">
<xsl:import href="functions/SeafServiceBusinessFunctions.xsl" />
<xsl:import href="generator/seaf-persistence/Beans.xsl" />
<xsl:output method="xml" indent="yes" />

<xsl:param name="persistence-merged-file-name" />
<xsl:param name="service-merged-file-name" />
<xsl:variable name="merged-persistence-file" select="document($persistence-merged-file-name)" />
<xsl:variable name="merged-service-file" select="document($service-merged-file-name)" />

<xsl:variable name="entities" select="$merged-persistence-file//per:entity" />
<xsl:variable name="relations" select="$merged-persistence-file//per:relation" />
<xsl:variable name="objects" select="$merged-service-file//svc:object" />

<xsl:template match="/">
	<jbg xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xsi:schemaLocation="http://jbg.sf.net/jbg http://jbg.sf.net/xsd/1.0/jbg.xsd">
		<beans to-string-properties="all" final-accessors="no">
			<xsl:apply-templates select="//svc:service-layer//svc:object" />
		</beans>
	</jbg>
</xsl:template>

<xsl:template match="svc:object">
	<xsl:variable name="svc-object" select="." />
	<xsl:variable name="full-type" select="svc:service-object-full-type(.)" />
	<xsl:variable name="class" select="seaf:type-class($full-type)" />
	<bean class="{$full-type}" serializable-with-id="1">
		<description>Service object <xsl:value-of select="$class"/>.</description>
	<xsl:variable name="object-fields" select="svc:object-fields(.,$merged-persistence-file)" />
	<xsl:for-each select="$object-fields">
		<xsl:call-template name="field-template">
			<xsl:with-param name="svc-object" select="$svc-object" />
		</xsl:call-template>
	</xsl:for-each>
	<xsl:apply-templates select="svc:relation">
		<xsl:with-param name="svc-object" select="$svc-object" />
	</xsl:apply-templates>
	<xsl:if test="@processing-type='criteria-based'">
		<property name="sortCriteria_" status="business-key">
			<sequence type="array">
				<bean-type type="PropertySort" />
			</sequence>
		</property>
		<additional-code>
			<xsl:value-of select="svc:property-enum-and-sort($svc-object)" />
		</additional-code>
	</xsl:if>
	</bean>
	<xsl:if test="$merged-service-file//svc:response-sequence[@paged='true' 
		and svc:service-object-full-type(.)=$full-type]">
		<bean class="{$full-type}PagedResult" serializable-with-id="1">
			<description>Paged result for service object <xsl:text />
				<xsl:value-of select="$class"/>.</description>
			<property name="pagedInfo" status="business-key">
				<description>Information on the page (subset) of result set being returned.</description>
				<bean-type type="net.sf.seaf.common.PagedInfo" />
			</property>
			<property name="values" status="business-key">
				<description>The page (subset) of result set.</description>
				<sequence type="array">
					<bean-type type="{$full-type}" />
				</sequence>
			</property>
		</bean>
	</xsl:if>
</xsl:template>

<xsl:template name="field-template">
<xsl:param name="svc-object" as="node()" />
	<xsl:variable name="field-name" select="@name" />
	<xsl:variable name="is-criteria" 
		select="$svc-object/@processing-type='criteria-based'" />
	<xsl:variable name="is-non-validating" 
		select="$svc-object/@processing-type='non-validating' 
			or $svc-object/@processing-type='criteria-based'" />
	<xsl:variable name="per-field" 
		select="svc:expand-code-table-field(.,$merged-persistence-file)" />
	<xsl:if test="not($per-field/@status='criteria-only') or $is-criteria">
	<property name="{$field-name}" status="business-key">
		<xsl:apply-templates select="per:description" />
		<xsl:choose>
		<xsl:when test="per:enum">
			<simple-type type="enum" class="{svc:enum-ref-full-type(per:enum/@name,$merged-service-file)}" />
		</xsl:when>
		<xsl:when test="per:binary">
			<sequence type="array">
				<simple-type type="byte" />
			</sequence>
		</xsl:when>
		<xsl:otherwise>
			<simple-type type="{per:field-simple-property-type($per-field)}" />
		</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="not($is-non-validating)">
			<xsl:variable name="svc-field" select="$svc-object//svc:field[@ref=$field-name]" />
			<xsl:call-template name="string-size-annotation-template">
				<xsl:with-param name="svc-field" select="$svc-field" />
				<xsl:with-param name="per-field" select="$per-field" />
			</xsl:call-template>
			<xsl:if test="$svc-field/@min">
				<annotation>@javax.validation.constraints.Min(<xsl:text />
					<xsl:value-of select="$svc-field/@min" />)</annotation>
			</xsl:if>
			<xsl:if test="$svc-field/@max">
				<annotation>@javax.validation.constraints.Max(<xsl:text />
					<xsl:value-of select="$svc-field/@max" />)</annotation>
			</xsl:if>
			<xsl:if test="not($svc-field/@optional)">
				<annotation>@javax.validation.constraints.NotNull</annotation>
			</xsl:if>
		</xsl:if>
	</property>
	<xsl:if test="$is-criteria">
	<property name="{seaf:operator-field-name($field-name)}" status="business-key">
		<description>Operator for field <xsl:value-of select="$field-name" />.</description>
		<simple-type type="enum" class="net.sf.seaf.common.Operator" />
	</property>
	</xsl:if>
	</xsl:if>
</xsl:template>

<xsl:template name="string-size-annotation-template">
<xsl:param name="svc-field" as="node()*" />
<xsl:param name="per-field" as="node()" />
	<xsl:if test="$per-field/per:string">
		<annotation>@javax.validation.constraints.Size(<xsl:text />
		<xsl:if test="svc:field-has-size-min($svc-field)">
			<xsl:text />min = <xsl:text />
			<xsl:value-of select="svc:field-size-min($svc-field)" />, <xsl:text />
		</xsl:if>
		<xsl:text />max = <xsl:text />
			<xsl:value-of select="svc:field-size-max($svc-field,$per-field)" />)</annotation>
	</xsl:if>
</xsl:template>

<xsl:template match="svc:relation">
<xsl:param name="svc-object" as="node()" />
	<xsl:variable name="per-relation" select="svc:per-relation(.,..,$merged-persistence-file)" />
	<xsl:variable name="per-relation-side" 
		select="svc:per-relation-side(.,..,$per-relation)" />
	<xsl:variable name="related-object-full-type" 
		select="svc:ref-service-object-full-type(.,$objects)" />
	<xsl:variable name="is-non-validating" 
		select="$svc-object/@processing-type='non-validating' 
			or $svc-object/@processing-type='criteria-based'" />
	<property name="{per:relation-property-name($per-relation-side,$per-relation)}" status="business-key">
		<xsl:choose>
		<xsl:when test="$per-relation/@type='one-to-many' and name($per-relation-side)='master'">
			<sequence type="array">
				<bean-type type="{$related-object-full-type}"/>
			</sequence>
		</xsl:when>
		<xsl:otherwise>
			<bean-type type="{$related-object-full-type}"/>
		</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="not($is-non-validating)">
			<xsl:if test="not(@optional)">
				<annotation>@javax.validation.constraints.NotNull</annotation>
			</xsl:if>
			<annotation>@javax.validation.Valid</annotation>
		</xsl:if>
	</property>
</xsl:template>

<xsl:template match="per:description">
	<description><xsl:value-of select="." /></description>
</xsl:template>

<xsl:function name="svc:property-enum-and-sort" as="xs:string">
	<xsl:param name="svc-object" as="node()" />
	<xsl:variable name="entity" 
		select="per:entity-ref($svc-object/@entity,$merged-persistence-file)" />
	<xsl:variable name="persistent-fields" 
		select="per:entity-fields($entity,$merged-persistence-file)
			[not(@status='transient' or @status='criteria-only' or @status='version')]" />
	<xsl:variable name="result">
	<xsl:text />

	/**
	 * Names of sortable properties.
	 */
	public enum Property {
		<xsl:text />
	<xsl:choose>
	<xsl:when test="$svc-object/svc:sort">
		<xsl:for-each select="$svc-object/svc:sort/svc:field/@ref">
			<xsl:variable name="name" select="." />
			<xsl:if test="empty($persistent-fields[@name=$name])">
				<xsl:message>Error generating sort property list.</xsl:message>
				<xsl:message terminate="yes" 
					select="concat('Field ',.,' not found on entity ',$svc-object/@entity,'.')" />
			</xsl:if>
			<xsl:value-of select="." />
			<xsl:if test="not(position()=last())">, </xsl:if>
		</xsl:for-each>
	</xsl:when>
	<xsl:otherwise>
		<xsl:value-of select="string-join($persistent-fields/@name,', ')" />
	</xsl:otherwise>
	</xsl:choose>
	}

	public static final class PropertySort extends net.sf.seaf.common.PropertySort&lt;Property> {
		public PropertySort() {
		}
		public PropertySort(Property property, net.sf.seaf.common.SortOrder order) {
			super(property, order);
		}
		public PropertySort(Property property) {
			super(property);
		}
		@Override
		public void setProperty(Property property) {
			super.setProperty(property);
		}
		private static final long serialVersionUID = 1L;
	}<xsl:text />
	</xsl:variable>
	<xsl:value-of select="$result"/>
</xsl:function>

</xsl:stylesheet>
